@@ -1,13 +1,16 @@
 #' Capture all Output
 #'
 #' This function silences all warnings, errors & messages and instead returns a list
-#' containing the results (if it didn't error) + the warning and error messages as
-#' character vectors.
+#' containing the results (if it didn't error), as well as the warnings, errors
+#' and messages and divergence signals as character vectors.
 #'
 #' @param expr (`expression`)\cr to be executed.
 #' @param remove (`list`)\cr optional list with elements `warnings`, `errors`,
 #'   `messages` which can be character vectors, which will be removed from the
 #'   results if specified.
+#' @param divergence (`list`)\cr optional list similar as `remove`, but these
+#'   character vectors will be moved to the `divergence` result and signal
+#'   that the fit did not converge.
 #'
 #' @return
 #' A list containing
@@ -16,9 +19,12 @@
 #' - `warnings`: `NULL` or a character vector if warnings were thrown.
 #' - `errors`: `NULL` or a string if an error was thrown.
 #' - `messages`: `NULL` or a character vector if messages were produced.
+#' - `divergence`: `NULL` or a character vector if divergence messages were caught.
 #'
 #' @keywords internal
-h_record_all_output <- function(expr, remove = list()) {
+h_record_all_output <- function(expr,
+                                remove = list(),
+                                divergence = list()) {
   # Note: We don't need to and cannot assert `expr` here.
   assert_list(remove)
   env <- new.env()
@@ -43,113 +49,17 @@ h_record_all_output <- function(expr, remove = list()) {
   )
   list(
     result = result,
-    warnings = setdiff(env$warning, remove$warnings),
-    errors = setdiff(env$error, remove$errors),
-    messages = setdiff(env$message, remove$messages)
+    warnings = setdiff(env$warning, c(remove$warnings, divergence$warnings)),
+    errors = setdiff(env$error, c(remove$errors, divergence$errors)),
+    messages = setdiff(env$message, c(remove$messages, divergence$messages)),
+    divergence = c(
+      intersect(env$warning, divergence$warnings),
+      intersect(env$error, divergence$errors),
+      intersect(env$message, divergence$messages)
+    )
   )
 }
 
-#' Get an approximate number of free cores.
-#'
-#' @description `r lifecycle::badge("deprecated")` use
-#' `parallelly::availableCores(omit = 1)` instead
-#'
-#' @return The approximate number of free cores, which is an integer between 1 and one less than
-#' the total cores.
-#'
-#' @details
-#' - This uses the maximum load average at 1, 5 and 15 minutes on Linux and Mac
-#' machines to approximate the number of busy cores. For Windows, the load percentage is
-#' multiplied with the total number of cores.
-#' - We then subtract this from the number of all detected cores. One additional core
-#' is not used for extra safety.
-#'
-#' @note If executed during a unit test and on CRAN then always returns 1 to avoid any
-#' parallelization.
-#'
-#' @export
-free_cores <- function() {
-  lifecycle::deprecate_warn(
-    "0.1.6", "free_cores()",
-    "parallelly::availableCores(omit = 'number of cores to reserve, e.g. 1')"
-  )
-  all_cores <- parallel::detectCores(all.tests = TRUE)
-  busy_cores <-
-    if (.Platform$OS.type == "windows") {
-      load_percent_string <- system("wmic cpu get loadpercentage", intern = TRUE)
-      # This gives e.g.: c("LoadPercentage", "10", "")
-      # So we just take the number here.
-      load_percent <- as.integer(min(load_percent_string[2L], 100))
-      if (test_int(load_percent, lower = 0, upper = 100)) {
-        ceiling(all_cores * load_percent / 100)
-      } else {
-        all_cores
-      }
-    } else if (.Platform$OS.type == "unix") {
-      uptime_string <- system("uptime", intern = TRUE)
-      # This gives e.g.:
-      # "11:00  up  1:57, 3 users, load averages: 2.71 2.64 2.62"
-      # Here we just want the last three numbers.
-      uptime_split <- strsplit(uptime_string, split = ",|\\s")[[1]] # Split at comma or white space.
-      uptime_split <- uptime_split[uptime_split != ""]
-      load_averages <- as.numeric(utils::tail(uptime_split, 3))
-      ceiling(max(load_averages))
-    }
-  assert_number(all_cores, lower = 1, finite = TRUE)
-  assert_number(busy_cores, lower = 0, upper = all_cores)
-  # For safety, we subtract 1 more core from all cores.
-  as.integer(max(1, all_cores - busy_cores - 1))
-}
-
-# covariance types ----
-
-# nolint start
-
-#' covariance type
-#'
-#' @format vector of supported covariance structures. `cov_type` for common time points covariance structures,
-#' `cov_type_spatial` for spatial covariance structures.
-#' @details
-#' abbreviation for covariance structures
-#' ## Common Covariance Structures
-#'
-#' | **Structure**     | **Description**                       | **Parameters**      | **\eqn{(i, j)} element**         |
-#' | ------------- |-------------------------------------------|:---------------|----------------------------------|
-#' | ad            | Ante-dependence                           | \eqn{m}        | \eqn{\sigma^{2}\prod_{k=i}^{j-1}\rho_{k}} |
-#' | adh           | Heterogeneous ante-dependence             | \eqn{2m-1}     | \eqn{\sigma_{i}\sigma_{j}\prod_{k=i}^{j-1}\rho_{k}} |
-#' | ar1           | First-order auto-regressive               | \eqn{2}        | \eqn{\sigma^{2}\rho^{\left \vert {i-j} \right \vert}} |
-#' | ar1h          | Heterogeneous first-order auto-regressive | \eqn{m+1}      | \eqn{\sigma_{i}\sigma_{j}\rho^{\left \vert {i-j} \right \vert}} |
-#' | cs            | Compound symmetry                         | \eqn{2}        | \eqn{\sigma^{2}\left[ \rho I(i \neq j)+I(i=j) \right]} |
-#' | csh           | Heterogeneous compound symmetry           | \eqn{m+1}      | \eqn{\sigma_{i}\sigma_{j}\left[ \rho I(i \neq j)+I(i=j) \right]} |
-#' | toep          | Toeplitz                                  | \eqn{m}        | \eqn{\sigma_{\left \vert {i-j} \right \vert +1}} |
-#' | toeph         | Heterogeneous Toeplitz                    | \eqn{2m-1}     | \eqn{\sigma_{i}\sigma_{j}\rho_{\left \vert {i-j} \right \vert}} |
-#' | us            | Unstructured                              | \eqn{m(m+1)/2} | \eqn{\sigma_{ij}} |
-#'
-#' where \eqn{i} and \eqn{j} denote \eqn{i}-th and \eqn{j}-th time points, respectively, out of total \eqn{m} time points, \eqn{1 \leq i, j \leq m}.
-#'
-#' Note the **ante-dependence** covariance structure in this package refers to homogeneous ante-dependence, while the ante-dependence covariance structure from SAS `PROC MIXED` refers to heterogeneous ante-dependence and the homogeneous version is not available in SAS.
-#'
-#' ## Spatial Covariance structures
-#'
-#' | **Structure**     | **Description**                       | **Parameters**      | **\eqn{(i, j)} element**         |
-#' | ------------- |-------------------------------------------|:---------------|----------------------------------|
-#' | sp_exp        | spatial exponential                       | \eqn{2}        | \eqn{\sigma^{2}\rho^{-d_{ij}}} |
-#'
-#' where \eqn{d_{ij}} denotes the Euclidean distance between time points \eqn{i} and \eqn{j}.
-#' @md
-#' @name covariance_types
-NULL
-
-# nolint end
-
-#' @describeIn covariance_types non-spatial covariance structure
-#' @format NULL
-cov_type <- c("us", "toep", "toeph", "ar1", "ar1h", "ad", "adh", "cs", "csh")
-#' @describeIn covariance_types spatial covariance structure
-#' @format NULL
-cov_type_spatial <- c("sp_exp")
-
-
 #' Trace of a Matrix
 #'
 #' @description Obtain the trace of a matrix if the matrix is diagonal, otherwise raise an error.
@@ -163,7 +73,7 @@ h_tr <- function(x) {
   if (nrow(x) != ncol(x)) {
     stop("x must be square matrix")
   }
-  sum(diag(x))
+  sum(Matrix::diag(x))
 }
 
 #' Split Control List
@@ -291,3 +201,149 @@ h_partial_fun_args <- function(fun, ..., additional_attr = list()) {
     ), additional_attr)
   )
 }
+
+#' Obtain Default Covariance Method
+#'
+#' @description Obtain the default covariance method depending on
+#' the degrees of freedom method used.
+#'
+#' @param method (`string`)\cr degrees of freedom method.
+#'
+#' @details The default covariance method is different for different degrees of freedom method.
+#' For "Satterthwaite" or "Between-Within", "Asymptotic" is returned.
+#' For "Kenward-Roger" only, "Kenward-Roger" is returned.
+#' For "Residual" only, "Empirical" is returned.
+#'
+#' @keywords internal
+h_get_cov_default <- function(method = c("Satterthwaite", "Kenward-Roger", "Residual", "Between-Within")) {
+  assert_string(method)
+  method <- match.arg(method)
+  switch(method,
+    "Residual" = "Empirical",
+    "Satterthwaite" = "Asymptotic",
+    "Kenward-Roger" = "Kenward-Roger",
+    "Between-Within" = "Asymptotic"
+  )
+}
+
+#' Complete `character` Vector Names From Values
+#'
+#' @param x (`character` or `list`)\cr value whose names should be completed
+#'   from element values.
+#'
+#' @return A named vector or list.
+#'
+#' @keywords internal
+fill_names <- function(x) {
+  n <- names(x)
+  is_unnamed <- if (is.null(n)) rep_len(TRUE, length(x)) else n == ""
+  names(x)[is_unnamed] <- x[is_unnamed]
+  x
+}
+
+#' Drop Items from an Indexible
+#'
+#' Drop elements from an indexible object (`vector`, `list`, etc.).
+#'
+#' @param x Any object that can be consumed by [seq_along()] and indexed by a
+#'   logical vector of the same length.
+#' @param n (`integer`)\cr the number of terms to drop.
+#'
+#' @return A subset of `x`.
+#'
+#' @keywords internal
+drop_elements <- function(x, n) {
+  x[seq_along(x) > n]
+}
+
+#' Ask for Confirmation on Large Visit Levels
+#'
+#' @description Ask the user for confirmation if there are too many visit levels
+#' for non-spatial covariance structure in interactive sessions.
+#'
+#' @param x (`numeric`)\cr number of visit levels.
+#'
+#' @keywords internal
+h_confirm_large_levels <- function(x) {
+  assert_count(x)
+  allowed_lvls <- x <= getOption("mmrm.max_visits", 100)
+  if (allowed_lvls) {
+    return(TRUE)
+  }
+  if (!interactive()) {
+    stop("Visit levels too large!", call. = FALSE)
+  }
+  proceed <- utils::askYesNo(
+    paste(
+      "Visit levels is possibly too large.",
+      "This requires large memory. Are you sure to continue?",
+      collapse = " "
+    )
+  )
+  if (!identical(proceed, TRUE)) {
+    stop("Visit levels too large!", call. = FALSE)
+  }
+  return(TRUE)
+}
+
+#' Default Value on NULL
+#' Return default value when first argument is NULL.
+#'
+#' @param x Object.
+#' @param y Object.
+#'
+#' @details If `x` is NULL, returns `y`. Otherwise return `x`.
+#'
+#' @keywords internal
+h_default_value <- function(x, y) {
+  if (is.null(x)) {
+    y
+  } else {
+    x
+  }
+}
+
+#' Convert Character to Factor Following Reference
+#'
+#' @param x (`character` or `factor`)\cr  input.
+#' @param ref (`factor`)\cr reference.
+#' @param var_name (`string`)\cr variable name of input `x`.
+#'
+#' @details Use `ref` to convert `x` into factor with the same levels.
+#' This is needed even if `x` and `ref` are both `character` because
+#' in `model.matrix` if `x` only has one level there could be errors.
+#'
+#' @keywords internal
+h_factor_ref <- function(x, ref, var_name = vname(x)) {
+  assert_multi_class(ref, c("character", "factor"))
+  assert_multi_class(x, c("character", "factor"))
+  # NA can be possible values
+  uni_values <- as.character(stats::na.omit(unique(x)))
+  # no NA in reference
+  uni_ref <- as.character(unique(ref))
+  assert_character(uni_values, .var.name = var_name)
+  assert_subset(uni_values, uni_ref, .var.name = var_name)
+  factor(x, levels = h_default_value(levels(ref), sort(uni_ref)))
+}
+
+#' Warn on na.action
+#' @keywords internal
+h_warn_na_action <- function() {
+  if (!identical(getOption("na.action"), "na.omit")) {
+    warning("na.action is always set to `na.omit` for `mmrm` fit!")
+  }
+}
+
+#' Validate mmrm Formula
+#' @param formula (`formula`)\cr to check.
+#'
+#' @details In mmrm models, `.` is not allowed as it introduces ambiguity of covariates
+#' to be used, so it is not allowed to be in formula.
+#'
+#' @keywords internal
+h_valid_formula <- function(formula) {
+  assert_formula(formula)
+  if ("." %in% all.vars(formula)) {
+    stop("`.` is not allowed in mmrm models!")
+  }
+}
