@@ -1,5 +1,18 @@
 # recover_data ----
 
+test_that("emmeans emits a message about mmrm registration on load", {
+  # Don't use skip_if_not_installed yet since it runs requireNamespace, which
+  # will itself load the package without testing startup.
+  skip_if(length(find.package("emmeans")) < 1)
+
+  # Detach emmeans in case it was loaded in session or previous test run
+  if (isNamespaceLoaded("emmeans")) {
+    unloadNamespace("emmeans")
+  }
+
+  expect_message(library(emmeans), "mmrm")
+})
+
 test_that("recover_data method works as expected", {
   skip_if_not_installed("emmeans", minimum_version = "1.6")
 
@@ -79,6 +92,28 @@ test_that("emmeans works as expected", {
   expect_equal(result_emmeans, expected_emmeans, tolerance = 1e-3)
 })
 
+test_that("emmeans works as expected for transformed variables", {
+  skip_if_not_installed("emmeans", minimum_version = "1.6")
+
+  fit <- mmrm(FEV1 ~ log(FEV1_BL) + AVISIT + ar1(AVISIT | USUBJID), data = fev_data)
+  result <- expect_silent(emmeans::emmeans(fit, ~AVISIT))
+  expect_class(result, "emmGrid")
+  result_emmeans <- as.data.frame(result)$emmean
+  expected_emmeans <- c(34.99641, 39.86572, 44.54411, 50.53663)
+  expect_equal(result_emmeans, expected_emmeans, tolerance = 1e-3)
+})
+
+test_that("emmeans works as expected for transformed variables and fixed effect is not visit", {
+  skip_if_not_installed("emmeans", minimum_version = "1.6")
+
+  fit <- mmrm(FEV1 ~ log(FEV1_BL) + ARMCD + ar1(AVISIT | USUBJID), data = fev_data)
+  result <- expect_silent(emmeans::emmeans(fit, ~ ARMCD | AVISIT))
+  expect_class(result, "emmGrid")
+  result_emmeans <- as.data.frame(result)$emmean
+  expected_emmeans <- c(40.41980, 44.78855, 40.41980, 44.78855, 40.41980, 44.78855, 40.41980, 44.78855)
+  expect_equal(result_emmeans, expected_emmeans, tolerance = 1e-3)
+})
+
 test_that("emmeans gives values close to what is expected", {
   skip_if_not_installed("emmeans", minimum_version = "1.6")
 
@@ -153,7 +188,7 @@ test_that("emmeans gives d.f. close to what is expected for weighted model", {
   skip_if_not_installed("emmeans", minimum_version = "1.6")
 
   formula <- FEV1 ~ RACE + SEX + ARMCD * AVISIT + us(AVISIT | USUBJID)
-  fev_data <- within(fev_data, AVISIT <- relevel(AVISIT, ref = "VIS4"))# nolint
+  fev_data <- within(fev_data, AVISIT <- relevel(AVISIT, ref = "VIS4")) # nolint
   fit <- mmrm(formula, fev_data, weights = fev_data$WEIGHT)
   result <- expect_silent(emmeans::emmeans(fit, pairwise ~ ARMCD * AVISIT))
   expect_class(result, "emm_list")
@@ -164,24 +199,29 @@ test_that("emmeans gives d.f. close to what is expected for weighted model", {
   expected_emmeans_df <- data.frame(
     ARMCD = factor(c("PBO", "TRT", "PBO", "TRT", "PBO", "TRT", "PBO", "TRT")),
     AVISIT = factor(c("VIS4", "VIS4", "VIS1", "VIS1", "VIS2", "VIS2", "VIS3", "VIS3"),
-                    levels = c("VIS4", "VIS1", "VIS2", "VIS3")),
+      levels = c("VIS4", "VIS1", "VIS2", "VIS3")
+    ),
     df = c(134, 132, 147, 142, 144, 143, 128, 128)
   )
   expect_equal(result_emmeans_df, expected_emmeans_df, tolerance = 1e-2)
 
   expected_contrasts_df <- data.frame(
-    contrast = c("PBO VIS4 - TRT VIS4", "PBO VIS4 - PBO VIS1", "PBO VIS4 - TRT VIS1",
-                 "PBO VIS4 - PBO VIS2", "PBO VIS4 - TRT VIS2", "PBO VIS4 - PBO VIS3",
-                 "PBO VIS4 - TRT VIS3", "TRT VIS4 - PBO VIS1", "TRT VIS4 - TRT VIS1",
-                 "TRT VIS4 - PBO VIS2", "TRT VIS4 - TRT VIS2", "TRT VIS4 - PBO VIS3",
-                 "TRT VIS4 - TRT VIS3", "PBO VIS1 - TRT VIS1", "PBO VIS1 - PBO VIS2",
-                 "PBO VIS1 - TRT VIS2", "PBO VIS1 - PBO VIS3", "PBO VIS1 - TRT VIS3",
-                 "TRT VIS1 - PBO VIS2", "TRT VIS1 - TRT VIS2", "TRT VIS1 - PBO VIS3",
-                 "TRT VIS1 - TRT VIS3", "PBO VIS2 - TRT VIS2", "PBO VIS2 - PBO VIS3",
-                 "PBO VIS2 - TRT VIS3", "TRT VIS2 - PBO VIS3", "TRT VIS2 - TRT VIS3",
-                 "PBO VIS3 - TRT VIS3"),
-    df = c(133, 148, 231, 158, 198, 157, 190, 223, 122, 196, 147, 170, 161, 142,
-           151, 257, 157, 251, 258, 135, 225, 168, 144, 160, 270, 264, 159, 129)
+    contrast = c(
+      "PBO VIS4 - TRT VIS4", "PBO VIS4 - PBO VIS1", "PBO VIS4 - TRT VIS1",
+      "PBO VIS4 - PBO VIS2", "PBO VIS4 - TRT VIS2", "PBO VIS4 - PBO VIS3",
+      "PBO VIS4 - TRT VIS3", "TRT VIS4 - PBO VIS1", "TRT VIS4 - TRT VIS1",
+      "TRT VIS4 - PBO VIS2", "TRT VIS4 - TRT VIS2", "TRT VIS4 - PBO VIS3",
+      "TRT VIS4 - TRT VIS3", "PBO VIS1 - TRT VIS1", "PBO VIS1 - PBO VIS2",
+      "PBO VIS1 - TRT VIS2", "PBO VIS1 - PBO VIS3", "PBO VIS1 - TRT VIS3",
+      "TRT VIS1 - PBO VIS2", "TRT VIS1 - TRT VIS2", "TRT VIS1 - PBO VIS3",
+      "TRT VIS1 - TRT VIS3", "PBO VIS2 - TRT VIS2", "PBO VIS2 - PBO VIS3",
+      "PBO VIS2 - TRT VIS3", "TRT VIS2 - PBO VIS3", "TRT VIS2 - TRT VIS3",
+      "PBO VIS3 - TRT VIS3"
+    ),
+    df = c(
+      133, 148, 231, 158, 198, 157, 190, 223, 122, 196, 147, 170, 161, 142,
+      151, 257, 157, 251, 258, 135, 225, 168, 144, 160, 270, 264, 159, 129
+    )
   )
   expect_equal(result_contrasts_df, expected_contrasts_df, tolerance = 1e-2)
 })
