@@ -1,39 +1,14 @@
-# h_covbeta_fun ----
-
-test_that("h_covbeta_fun works as expected", {
-  model <- get_mmrm_tmb()
-  result <- expect_silent(h_covbeta_fun(model))
-  expect_function(result, args = "theta")
-  value <- result(model$theta_est)
-  expected_value <- model$beta_vcov
-  expect_equal(value, expected_value, ignore_attr = TRUE)
-  other_value <- result(model$theta_est * 1.1)
-  expect_false(identical(value, other_value))
-})
-
-# h_jac_col_as_matrix ----
-
-test_that("h_jac_col_as_matrix works as expected", {
-  jac_matrix <- matrix(1:81, 9, 9)
-  result <- expect_silent(h_jac_col_as_matrix(jac_matrix, 5))
-  expected <- matrix(37:45, 3, 3)
-  expect_identical(result, expected)
-})
-
 # h_jac_list ----
 
 test_that("h_jac_list works as expected", {
-  # Take a very simple function such that we know what are the derivatives.
-  covbeta_fun <- function(theta) {
-    matrix(1:4, 2, 2) + matrix(5:8, 2, 2) * theta
-  }
-  theta_est <- rep(0, 4) # Does not matter here.
-  result <- expect_silent(h_jac_list(covbeta_fun, theta_est))
-  expect_list(result, len = 4L)
-  expect_equal(result[[1L]], matrix(c(5, 0, 0, 0), 2, 2))
-  expect_equal(result[[2L]], matrix(c(0, 6, 0, 0), 2, 2))
-  expect_equal(result[[3L]], matrix(c(0, 0, 7, 0), 2, 2))
-  expect_equal(result[[4L]], matrix(c(0, 0, 0, 8), 2, 2))
+  # Take a spatial exponential because the number of parameters is small.
+  fit <- get_mmrm_spatial()
+  theta_est <- c(4, 0)
+  beta_vcov <- matrix(c(1, 0, 0, 1), ncol = 2)
+  result <- expect_silent(h_jac_list(fit$tmb_data, theta_est, beta_vcov))
+  expect_list(result, len = 2L)
+  expect_equal(result[[1L]], matrix(c(6.037242, 2.893871, 2.893871, 2.893871), 2, 2), tolerance = 1e-4)
+  expect_equal(result[[2L]], matrix(c(1.495479, 0.744836, 0.744836, 0.744836), 2, 2), tolerance = 1e-4)
 })
 
 # h_quad_form_vec ----
@@ -80,20 +55,6 @@ test_that("h_gradient works as expected", {
   expect_equal(result, expected)
 })
 
-# h_df_1d_list ----
-
-test_that("h_df_1d_list works as expected", {
-  result <- expect_silent(h_df_1d_list(est = 5, var = 4, v_num = 1, v_denom = 2))
-  expected <- list(
-    est = 5,
-    se = 2,
-    df = 1 / 2,
-    t_stat = 5 / 2,
-    p_val = 2 * stats::pt(q = 5 / 2, df = 1 / 2, lower.tail = FALSE)
-  )
-  expect_equal(result, expected)
-})
-
 # h_df_1d_sat ----
 
 test_that("h_df_1d_sat works as expected", {
@@ -158,19 +119,6 @@ test_that("h_md_denom_df works as expected when one t-statistic is 2 or smaller"
   expect_false(identical(h_md_denom_df(c(2.1, 5, 10, 15)), 2))
 })
 
-# h_df_md_list ----
-
-test_that("h_df_md_list works as expected", {
-  result <- expect_silent(h_df_md_list(f_stat = 0.38, num_df = 1, denom_df = 166))
-  expected <- list(
-    num_df = 1,
-    denom_df = 166,
-    f_stat = 0.38,
-    p_val = 0.5406
-  )
-  expect_equal(result, expected, tolerance = 1e-2)
-})
-
 # h_df_md_from_1d ----
 
 test_that("h_df_md_from_1d works as expected", {
